@@ -20,12 +20,18 @@
 
 #include "src/catalog/catalog_set.cpp"
 
+#include "src/catalog/default/default_schemas.cpp"
+
+#include "src/catalog/default/default_views.cpp"
+
 #include "src/catalog/dependency_manager.cpp"
 
 #include "src/common/checksum.cpp"
 
 #include "src/common/constants.cpp"
 
+#include "src/common/crypto/md5.cpp"
+
 #include "src/common/enums/catalog_type.cpp"
 
 #include "src/common/enums/expression_type.cpp"
@@ -42,6 +48,8 @@
 
 #include "src/common/exception.cpp"
 
+#include "src/common/exception_format_value.cpp"
+
 #include "src/common/file_buffer.cpp"
 
 #include "src/common/file_system.cpp"
@@ -68,6 +76,8 @@
 
 #include "src/common/string_util.cpp"
 
+#include "src/common/tree_renderer.cpp"
+
 #include "src/common/types.cpp"
 
 #include "src/common/types/chunk_collection.cpp"
@@ -146,9 +156,3 @@
 
 #include "src/execution/expression_executor/execute_cast.cpp"
 
-#include "src/execution/expression_executor/execute_comparison.cpp"
-
-#include "src/execution/expression_executor/execute_conjunction.cpp"
-
-#include "src/execution/expression_executor/execute_constant.cpp"
-
