@@ -3,6 +3,7 @@
 #include "duckdb/catalog/catalog_entry/schema_catalog_entry.hpp"
 #include "duckdb/common/exception.hpp"
 #include "duckdb/common/serializer.hpp"
+#include "duckdb/parser/parsed_data/alter_table_info.hpp"
 #include "duckdb/parser/parsed_data/create_view_info.hpp"
 #include "duckdb/common/limits.hpp"
 
@@ -17,6 +18,7 @@ void ViewCatalogEntry::Initialize(CreateViewInfo *info) {
 	this->types = info->types;
 	this->temporary = info->temporary;
 	this->sql = info->sql;
+	this->internal = info->internal;
 }
 
 ViewCatalogEntry::ViewCatalogEntry(Catalog *catalog, SchemaCatalogEntry *schema, CreateViewInfo *info)
@@ -24,7 +26,28 @@ ViewCatalogEntry::ViewCatalogEntry(Catalog *catalog, SchemaCatalogEntry *schema,
 	Initialize(info);
 }
 
+unique_ptr<CatalogEntry> ViewCatalogEntry::AlterEntry(ClientContext &context, AlterInfo *info) {
+	if (internal) {
+		throw CatalogException("Cannot use ALTER VIEW to alter a system view");
+	}
+	if (info->type != AlterType::ALTER_VIEW) {
+		throw CatalogException("Can only modify view with ALTER VIEW statement");
+	}
+	auto view_info = (AlterViewInfo *)info;
+	switch (view_info->alter_view_type) {
+	case AlterViewType::RENAME_VIEW: {
+		auto rename_info = (RenameViewInfo *)view_info;
+		auto copied_view = Copy(context);
+		copied_view->name = rename_info->new_view_name;
+		return copied_view;
+	}
+	default:
+		throw InternalException("Unrecognized alter view type!");
+	}
+}
+
 void ViewCatalogEntry::Serialize(Serializer &serializer) {
+	assert(!internal);
 	serializer.WriteString(schema->name);
 	serializer.WriteString(name);
 	serializer.WriteString(sql);
@@ -45,7 +68,7 @@ unique_ptr<CreateViewInfo> ViewCatalogEntry::Deserialize(Deserializer &source) {
 	info->schema = source.Read<string>();
 	info->view_name = source.Read<string>();
 	info->sql = source.Read<string>();
-	info->query = QueryNode::Deserialize(source);
+	info->query = SelectStatement::Deserialize(source);
 	auto alias_count = source.Read<uint32_t>();
 	for (uint32_t i = 0; i < alias_count; i++) {
 		info->aliases.push_back(source.Read<string>());
@@ -65,6 +88,7 @@ string ViewCatalogEntry::ToSQL() {
 }
 
 unique_ptr<CatalogEntry> ViewCatalogEntry::Copy(ClientContext &context) {
+	assert(!internal);
 	auto create_info = make_unique<CreateViewInfo>(schema->name, name);
 	create_info->query = query->Copy();
 	for (idx_t i = 0; i < aliases.size(); i++) {
