@@ -7,7 +7,8 @@
 namespace duckdb {
 using namespace std;
 
-template <class OP> static Value templated_binary_operation(const Value &left, const Value &right) {
+template<class OP>
+static Value binary_value_operation(const Value &left, const Value &right) {
 	auto left_type = left.type();
 	auto right_type = right.type();
 	LogicalType result_type = left_type;
@@ -15,15 +16,41 @@ template <class OP> static Value templated_binary_operation(const Value &left, c
 		result_type = LogicalType::MaxLogicalType(left.type(), right.type());
 		Value left_cast = left.CastAs(result_type);
 		Value right_cast = right.CastAs(result_type);
-		return templated_binary_operation<OP>(left_cast, right_cast);
+		return binary_value_operation<OP>(left_cast, right_cast);
 	}
 	if (left.is_null || right.is_null) {
 		return Value().CastAs(result_type);
 	}
 	if (TypeIsIntegral(result_type.InternalType())) {
+		hugeint_t left_hugeint;
+		hugeint_t right_hugeint;
+		switch(result_type.InternalType()) {
+		case PhysicalType::INT8:
+			left_hugeint = Hugeint::Convert(left.value_.tinyint);
+			right_hugeint = Hugeint::Convert(right.value_.tinyint);
+			break;
+		case PhysicalType::INT16:
+			left_hugeint = Hugeint::Convert(left.value_.smallint);
+			right_hugeint = Hugeint::Convert(right.value_.smallint);
+			break;
+		case PhysicalType::INT32:
+			left_hugeint = Hugeint::Convert(left.value_.integer);
+			right_hugeint = Hugeint::Convert(right.value_.integer);
+			break;
+		case PhysicalType::INT64:
+			left_hugeint = Hugeint::Convert(left.value_.bigint);
+			right_hugeint = Hugeint::Convert(right.value_.bigint);
+			break;
+		case PhysicalType::INT128:
+			left_hugeint = left.value_.hugeint;
+			right_hugeint = right.value_.hugeint;
+			break;
+		default:
+			throw NotImplementedException("Unimplemented type for value binary op");
+		}
 		// integer addition
 		return Value::Numeric(result_type, OP::template Operation<hugeint_t, hugeint_t, hugeint_t>(
-		                                       left.GetValue<hugeint_t>(), right.GetValue<hugeint_t>()));
+		                                       left_hugeint, right_hugeint));
 	} else if (result_type.InternalType() == PhysicalType::FLOAT) {
 		return Value::FLOAT(
 		    OP::template Operation<float, float, float>(left.GetValue<float>(), right.GetValue<float>()));
@@ -39,39 +66,31 @@ template <class OP> static Value templated_binary_operation(const Value &left, c
 // Numeric Operations
 //===--------------------------------------------------------------------===//
 Value ValueOperations::Add(const Value &left, const Value &right) {
-	return templated_binary_operation<duckdb::AddOperator>(left, right);
+	return binary_value_operation<duckdb::AddOperator>(left, right);
 }
 
 Value ValueOperations::Subtract(const Value &left, const Value &right) {
-	return templated_binary_operation<duckdb::SubtractOperator>(left, right);
+	return binary_value_operation<duckdb::SubtractOperator>(left, right);
 }
 
 Value ValueOperations::Multiply(const Value &left, const Value &right) {
-	return templated_binary_operation<duckdb::MultiplyOperator>(left, right);
+	return binary_value_operation<duckdb::MultiplyOperator>(left, right);
 }
 
 Value ValueOperations::Modulo(const Value &left, const Value &right) {
 	if (right == 0) {
-		return templated_binary_operation<duckdb::ModuloOperator>(left, Value(right.type()));
+		return Value(right.type());
 	} else {
-		return templated_binary_operation<duckdb::ModuloOperator>(left, right);
+		return binary_value_operation<duckdb::ModuloOperator>(left, right);
 	}
 }
 
 Value ValueOperations::Divide(const Value &left, const Value &right) {
 	if (right == 0) {
-		return templated_binary_operation<duckdb::DivideOperator>(left, Value(right.type()));
+		return Value(right.type());
 	} else {
-		return templated_binary_operation<duckdb::DivideOperator>(left, right);
+		return binary_value_operation<duckdb::DivideOperator>(left, right);
 	}
 }
 
-// Value ValueOperations::Min(const Value &left, const Value &right) {
-// 	return templated_binary_operation<duckdb::Min, true>(left, right);
-// }
-
-// Value ValueOperations::Max(const Value &left, const Value &right) {
-// 	return templated_binary_operation<duckdb::Max, true>(left, right);
-// }
-
 } // namespace duckdb
