@@ -7,7 +7,7 @@
 #include "duckdb/planner/expression_binder.hpp"
 #include "duckdb/storage/buffer_manager.hpp"
 #include "duckdb/storage/storage_manager.hpp"
-
+#include "duckdb/common/enums/output_type.hpp"
 #include <cctype>
 
 namespace duckdb {
@@ -24,8 +24,10 @@ static void pragma_enable_profiling_assignment(ClientContext &context, vector<Va
 		context.profiler.automatic_print_format = ProfilerPrintFormat::JSON;
 	} else if (assignment == "query_tree") {
 		context.profiler.automatic_print_format = ProfilerPrintFormat::QUERY_TREE;
+	} else if (assignment == "query_tree_optimizer") {
+		context.profiler.automatic_print_format = ProfilerPrintFormat::QUERY_TREE_OPTIMIZER;
 	} else {
-		throw ParserException("Unrecognized print format %s, supported formats: [json, query_tree]", assignment);
+		throw ParserException("Unrecognized print format %s, supported formats: [json, query_tree, query_tree_optimizer]", assignment);
 	}
 	context.profiler.Enable();
 }
@@ -107,6 +109,10 @@ static void pragma_enable_force_parallelism(ClientContext &context, vector<Value
 	context.force_parallelism = true;
 }
 
+static void pragma_enable_force_index_join(ClientContext &context, vector<Value> parameters) {
+	context.force_index_join = true;
+}
+
 static void pragma_disable_force_parallelism(ClientContext &context, vector<Value> parameters) {
 	context.force_parallelism = false;
 }
@@ -122,16 +128,27 @@ static void pragma_log_query_path(ClientContext &context, vector<Value> paramete
 }
 
 static void pragma_explain_output(ClientContext &context, vector<Value> parameters) {
-	string val = parameters[0].ToString();
-	if (val == "optimized") {
-		context.explain_output_optimized_only = true;
-	} else if (val == "all") {
-		context.explain_output_optimized_only = true;
+	string val = StringUtil::Lower(parameters[0].ToString());
+	if (val == "all") {
+		context.explain_output_type = ExplainOutputType::ALL;
+	} else if (val == "optimized_only") {
+		context.explain_output_type = ExplainOutputType::OPTIMIZED_ONLY;
+	} else if (val == "physical_only") {
+		context.explain_output_type = ExplainOutputType::PHYSICAL_ONLY;
 	} else {
-		throw ParserException("Expected PRAGMA explain_output={optimized, all}");
+		throw ParserException("Unrecognized output type '%s', expected either ALL, OPTIMIZED_ONLY or PHYSICAL_ONLY",
+		                      val);
 	}
 }
 
+static void pragma_enable_optimizer(ClientContext &context, vector<Value> parameters) {
+	context.enable_optimizer = true;
+}
+
+static void pragma_disable_optimizer(ClientContext &context, vector<Value> parameters) {
+	context.enable_optimizer = false;
+}
+
 void PragmaFunctions::RegisterFunction(BuiltinFunctions &set) {
 	register_enable_profiling(set);
 
@@ -161,8 +178,13 @@ void PragmaFunctions::RegisterFunction(BuiltinFunctions &set) {
 	set.AddFunction(PragmaFunction::PragmaStatement("force_parallelism", pragma_enable_force_parallelism));
 	set.AddFunction(PragmaFunction::PragmaStatement("disable_force_parallelism", pragma_disable_force_parallelism));
 
+	set.AddFunction(PragmaFunction::PragmaStatement("enable_optimizer", pragma_enable_optimizer));
+	set.AddFunction(PragmaFunction::PragmaStatement("disable_optimizer", pragma_disable_optimizer));
+
 	set.AddFunction(PragmaFunction::PragmaAssignment("log_query_path", pragma_log_query_path, LogicalType::VARCHAR));
 	set.AddFunction(PragmaFunction::PragmaAssignment("explain_output", pragma_explain_output, LogicalType::VARCHAR));
+
+	set.AddFunction(PragmaFunction::PragmaStatement("force_index_join", pragma_enable_force_index_join));
 }
 
 idx_t ParseMemoryLimit(string arg) {
