@@ -6,7 +6,7 @@ using namespace std;
 
 namespace duckdb {
 
-static bool is_ascii(const char *input, idx_t n) {
+bool StripAccentsFun::IsAscii(const char *input, idx_t n) {
 	for (idx_t i = 0; i < n; i++) {
 		if (input[i] & 0x80) {
 			// non-ascii character
@@ -22,7 +22,7 @@ static void strip_accents_function(DataChunk &args, ExpressionState &state, Vect
 	UnaryExecutor::Execute<string_t, string_t, true>(args.data[0], result, args.size(), [&](string_t input) {
 		auto input_data = input.GetData();
 		auto input_length = input.GetSize();
-		if (is_ascii(input_data, input_length)) {
+		if (StripAccentsFun::IsAscii(input_data, input_length)) {
 			return input;
 		}
 		// non-ascii, perform collation
