@@ -20,22 +20,8 @@ string_t SubstringFun::substring_ascii_only(Vector &result, const char *input_da
 
 string_t SubstringFun::substring_scalar_function(Vector &result, string_t input, int offset, int length,
                                                  unique_ptr<char[]> &output, idx_t &current_len) {
-	// reduce offset by one because SQL starts counting at 1
-	offset--;
-	if (offset < 0 || length < 0) {
-		throw Exception("SUBSTRING cannot handle negative offsets");
-	}
-	auto input_data = input.GetData();
-	auto input_size = input.GetSize();
-
-	// check if there is any non-ascii
-	bool ascii_only = true;
-	int ascii_end = min(offset + length + 1, (int)input_size);
-	for (int i = 0; i < ascii_end; i++) {
-		if (input_data[i] & 0x80) {
-			ascii_only = false;
-			break;
-		}
+	if (length < 0) {
+		throw Exception("SUBSTRING cannot handle negative lengths");
 	}
 
 	if (length == 0) {
@@ -44,8 +30,37 @@ string_t SubstringFun::substring_scalar_function(Vector &result, string_t input,
 		return result_string;
 	}
 
+	// reduce offset by one because SQL starts counting at 1
+	// special case: if offset is zero, reduce length by 1 to avoid copying uninitialised memory
+	if (offset > 0) {
+		--offset;
+	} else if (offset == 0) {
+		--length;
+	}
+	auto input_data = input.GetData();
+	auto input_size = input.GetSize();
+
+	// check if there is any non-ascii
+	bool ascii_only = true;
+	int ascii_begin = 0;
+	int ascii_end = (int)input_size;
+	if (offset < 0) {
+		ascii_begin = MaxValue<int>(input_size + offset, ascii_begin);
+	} else {
+		// scan one further to check for a grapheme cluster
+		ascii_end = MinValue<int>(offset + length + 1, ascii_end);
+	}
+	for (auto i = ascii_begin; i < ascii_end; ++i) {
+		if (input_data[i] & 0x80) {
+			ascii_only = false;
+			break;
+		}
+	}
+
 	if (ascii_only) {
-		// ascii only
+		if (offset < 0) {
+			offset = ascii_begin;
+		}
 		length = MinValue<int>(offset + length, input_size);
 		if (offset >= length) {
 			return string_t((uint32_t)0);
@@ -61,6 +76,15 @@ string_t SubstringFun::substring_scalar_function(Vector &result, string_t input,
 		output = unique_ptr<char[]>{new char[required_len]};
 	}
 
+	// if we are slicing Unicode, get the positive offset by computing the length
+	if (offset < 0) {
+		utf8proc_grapheme_callback(input_data, input_size, [&](size_t start, size_t end) {
+			++offset;
+			return true;
+		});
+		offset = MaxValue<int>(offset, 0);
+	}
+
 	// use grapheme iterator to iterate over the characters
 	int current_offset = 0;
 	int output_size = 0;
