@@ -4,7 +4,8 @@
 #include "duckdb/catalog/catalog_entry/schema_catalog_entry.hpp"
 #include "duckdb/catalog/catalog_entry/table_catalog_entry.hpp"
 #include "duckdb/common/exception.hpp"
-#include "duckdb/transaction/transaction.hpp"
+#include "duckdb/catalog/catalog_entry/index_catalog_entry.hpp"
+#include "duckdb/storage/data_table.hpp"
 
 #include <algorithm>
 
@@ -47,10 +48,10 @@ unique_ptr<FunctionOperatorData> sqlite_master_init(ClientContext &context, cons
 	auto result = make_unique<SQLiteMasterData>();
 
 	// scan all the schemas for tables and views and collect them
-	auto &transaction = Transaction::GetTransaction(context);
-	Catalog::GetCatalog(context).schemas->Scan(transaction, [&](CatalogEntry *entry) {
+	Catalog::GetCatalog(context).schemas->Scan(context, [&](CatalogEntry *entry) {
 		auto schema = (SchemaCatalogEntry *)entry;
-		schema->tables.Scan(transaction, [&](CatalogEntry *entry) { result->entries.push_back(entry); });
+		schema->tables.Scan(context, [&](CatalogEntry *entry) { result->entries.push_back(entry); });
+		schema->indexes.Scan(context, [&](CatalogEntry *entry) { result->entries.push_back(entry); });
 	});
 
 	return move(result);
@@ -63,17 +64,19 @@ void sqlite_master(ClientContext &context, const FunctionData *bind_data, Functi
 		// finished returning values
 		return;
 	}
-	idx_t next = min(data.offset + STANDARD_VECTOR_SIZE, (idx_t)data.entries.size());
-	output.SetCardinality(next - data.offset);
 
 	// start returning values
 	// either fill up the chunk or return all the remaining columns
-	for (idx_t i = data.offset; i < next; i++) {
-		auto index = i - data.offset;
-		auto &entry = data.entries[i];
+	idx_t count = 0;
+	while (data.offset < data.entries.size() && count < STANDARD_VECTOR_SIZE) {
+		auto &entry = data.entries[data.offset++];
+		if (entry->internal) {
+			continue;
+		}
 
 		// return values:
 		// "type", PhysicalType::VARCHAR
+		string table_name = entry->name;
 		const char *type_str;
 		switch (entry->type) {
 		case CatalogType::TABLE_ENTRY:
@@ -88,20 +91,27 @@ void sqlite_master(ClientContext &context, const FunctionData *bind_data, Functi
 		case CatalogType::VIEW_ENTRY:
 			type_str = "view";
 			break;
+		case CatalogType::INDEX_ENTRY: {
+			auto &index = (IndexCatalogEntry &) *entry;
+			table_name = index.info->table;
+			type_str = "index";
+			break;
+		}
 		default:
 			type_str = "unknown";
 		}
-		output.SetValue(0, index, Value(type_str));
+		output.SetValue(0, count, Value(type_str));
 		// "name", PhysicalType::VARCHAR
-		output.SetValue(1, index, Value(entry->name));
+		output.SetValue(1, count, Value(entry->name));
 		// "tbl_name", PhysicalType::VARCHAR
-		output.SetValue(2, index, Value(entry->name));
+		output.SetValue(2, count, Value(table_name));
 		// "rootpage", PhysicalType::INT32
-		output.SetValue(3, index, Value::INTEGER(0));
+		output.SetValue(3, count, Value::INTEGER(0));
 		// "sql", PhysicalType::VARCHAR
-		output.SetValue(4, index, Value(entry->ToSQL()));
+		output.SetValue(4, count, Value(entry->ToSQL()));
+		count++;
 	}
-	data.offset = next;
+	output.SetCardinality(count);
 }
 
 void SQLiteMaster::RegisterFunction(BuiltinFunctions &set) {
