@@ -81,6 +81,10 @@ struct LeftFun {
 	static void RegisterFunction(BuiltinFunctions &set);
 };
 
+struct MD5Fun {
+	static void RegisterFunction(BuiltinFunctions &set);
+};
+
 struct NFCNormalizeFun {
 	static ScalarFunction GetFunction();
 	static void RegisterFunction(BuiltinFunctions &set);
@@ -144,4 +148,8 @@ struct UnicodeFun {
 	static void RegisterFunction(BuiltinFunctions &set);
 };
 
+struct StringSplitFun {
+	static void RegisterFunction(BuiltinFunctions &set);
+};
+
 } // namespace duckdb
