@@ -69,6 +69,11 @@ void DuckDB::Configure(DBConfig &new_config) {
 	} else {
 		config.file_system = make_unique<FileSystem>();
 	}
+	if (config.maximum_memory == (idx_t)-1) {
+		config.maximum_memory = config.file_system->GetAvailableMemory() * 8 / 10;
+	} else {
+		config.maximum_memory = new_config.maximum_memory;
+	}
 	config.checkpoint_only = new_config.checkpoint_only;
 	config.checkpoint_wal_size = new_config.checkpoint_wal_size;
 	config.use_direct_io = new_config.use_direct_io;
