@@ -73,8 +73,13 @@ template <class T> void WriteData(duckdb_result *out, ChunkCollection &source, i
 	auto target = (T *)out->columns[col].data;
 	for (auto &chunk : source.chunks) {
 		auto source = FlatVector::GetData<T>(chunk->data[col]);
-		for (idx_t k = 0; k < chunk->size(); k++) {
-			target[row++] = source[k];
+		auto &nullmask = FlatVector::Nullmask(chunk->data[col]);
+
+		for (idx_t k = 0; k < chunk->size(); k++, row++) {
+			if (nullmask[k]) {
+				continue;
+			}
+			target[row] = source[k];
 		}
 	}
 }
