@@ -13,9 +13,10 @@ namespace duckdb {
 using namespace std;
 
 unique_ptr<BoundTableRef> Binder::Bind(BaseTableRef &ref) {
+	QueryErrorContext error_context(root_statement, ref.query_location);
 	// CTEs and views are also referred to using BaseTableRefs, hence need to distinguish here
 	// check if the table name refers to a CTE
-	auto cte = FindCTE(ref.table_name);
+	auto cte = FindCTE(ref.table_name, ref.table_name == alias);
 	if (cte) {
 		// Check if there is a CTE binding in the BindContext
 		auto ctebinding = bind_context.GetCTEBinding(ref.table_name);
@@ -24,6 +25,13 @@ unique_ptr<BoundTableRef> Binder::Bind(BaseTableRef &ref) {
 			SubqueryRef subquery(cte->query->Copy());
 			subquery.alias = ref.alias.empty() ? ref.table_name : ref.alias;
 			subquery.column_name_alias = cte->aliases;
+			for (idx_t i = 0; i < ref.column_name_alias.size(); i++) {
+				if (i < subquery.column_name_alias.size()) {
+					subquery.column_name_alias[i] = ref.column_name_alias[i];
+				} else {
+					subquery.column_name_alias.push_back(ref.column_name_alias[i]);
+				}
+			}
 			return Bind(subquery);
 		} else {
 			// There is a CTE binding in the BindContext.
@@ -31,21 +39,23 @@ unique_ptr<BoundTableRef> Binder::Bind(BaseTableRef &ref) {
 			auto index = GenerateTableIndex();
 			auto result = make_unique<BoundCTERef>(index, ctebinding->index);
 			auto b = ctebinding;
+			auto alias = ref.alias.empty() ? ref.table_name : ref.alias;
+			auto names = BindContext::AliasColumnNames(alias, b->names, ref.column_name_alias);
 
-			bind_context.AddGenericBinding(index, ref.alias.empty() ? ref.table_name : ref.alias, b->names, b->types);
+			bind_context.AddGenericBinding(index, alias, names, b->types);
 			// Update references to CTE
 			auto cteref = bind_context.cte_references[ref.table_name];
 			(*cteref)++;
 
 			result->types = b->types;
-			result->bound_columns = b->names;
+			result->bound_columns = move(names);
 			return move(result);
 		}
 	}
 	// not a CTE
 	// extract a table or view from the catalog
-	auto table_or_view =
-	    Catalog::GetCatalog(context).GetEntry(context, CatalogType::TABLE_ENTRY, ref.schema_name, ref.table_name);
+	auto table_or_view = Catalog::GetCatalog(context).GetEntry(context, CatalogType::TABLE_ENTRY, ref.schema_name,
+	                                                           ref.table_name, false, error_context);
 	switch (table_or_view->type) {
 	case CatalogType::TABLE_ENTRY: {
 		// base table: create the BoundBaseTableRef node
@@ -54,38 +64,46 @@ unique_ptr<BoundTableRef> Binder::Bind(BaseTableRef &ref) {
 
 		auto scan_function = TableScanFunction::GetFunction();
 		auto bind_data = make_unique<TableScanBindData>(table);
+		auto alias = ref.alias.empty() ? ref.table_name : ref.alias;
 		vector<LogicalType> table_types;
 		vector<string> table_names;
 		for (auto &col : table->columns) {
 			table_types.push_back(col.type);
 			table_names.push_back(col.name);
 		}
+		table_names = BindContext::AliasColumnNames(alias, table_names, ref.column_name_alias);
 
 		auto logical_get =
 		    make_unique<LogicalGet>(table_index, scan_function, move(bind_data), table_types, table_names);
-		auto alias = ref.alias.empty() ? ref.table_name : ref.alias;
-		bind_context.AddBaseTable(table_index, alias, move(table_names), move(table_types), table->name_map,
-		                          *logical_get);
+		bind_context.AddBaseTable(table_index, alias, move(table_names), move(table_types), *logical_get);
 		return make_unique_base<BoundTableRef, BoundBaseTableRef>(table, move(logical_get));
 	}
 	case CatalogType::VIEW_ENTRY: {
 		// the node is a view: get the query that the view represents
 		auto view_catalog_entry = (ViewCatalogEntry *)table_or_view;
+		// We need to use a new binder for the view that doesn't reference any CTEs
+		// defined for this binder so there are no collisions between the CTEs defined
+		// for the view and for the current query
+		bool inherit_ctes = false;
+		Binder view_binder(context, this, inherit_ctes);
 		SubqueryRef subquery(view_catalog_entry->query->Copy());
 		subquery.alias = ref.alias.empty() ? ref.table_name : ref.alias;
-		subquery.column_name_alias = view_catalog_entry->aliases;
+		subquery.column_name_alias =
+		    BindContext::AliasColumnNames(subquery.alias, view_catalog_entry->aliases, ref.column_name_alias);
 		// bind the child subquery
-		auto bound_child = Bind(subquery);
+		auto bound_child = view_binder.Bind(subquery);
 		assert(bound_child->type == TableReferenceType::SUBQUERY);
 		// verify that the types and names match up with the expected types and names
 		auto &bound_subquery = (BoundSubqueryRef &)*bound_child;
 		if (bound_subquery.subquery->types != view_catalog_entry->types) {
 			throw BinderException("Contents of view were altered: types don't match!");
 		}
+		bind_context.AddSubquery(bound_subquery.subquery->GetRootIndex(), subquery.alias, subquery,
+		                         *bound_subquery.subquery);
 		return bound_child;
 	}
 	default:
-		throw NotImplementedException("Catalog entry type");
+		throw InternalException("Catalog entry type");
 	}
 }
 } // namespace duckdb
