@@ -1,4 +1,5 @@
 #include "duckdb/transaction/rollback_state.hpp"
+#include "duckdb/transaction/append_info.hpp"
 #include "duckdb/transaction/delete_info.hpp"
 #include "duckdb/transaction/update_info.hpp"
 
@@ -7,6 +8,7 @@
 
 #include "duckdb/catalog/catalog_entry.hpp"
 #include "duckdb/catalog/catalog_set.hpp"
+#include "duckdb/storage/data_table.hpp"
 
 namespace duckdb {
 using namespace std;
@@ -20,6 +22,12 @@ void RollbackState::RollbackEntry(UndoFlags type, data_ptr_t data) {
 		catalog_entry->set->Undo(catalog_entry);
 		break;
 	}
+	case UndoFlags::INSERT_TUPLE: {
+		auto info = (AppendInfo *)data;
+		// revert the append in the base table
+		info->table->RevertAppend(info->start_row, info->count);
+		break;
+	}
 	case UndoFlags::DELETE_TUPLE: {
 		auto info = (DeleteInfo *)data;
 		// reset the deleted flag on rollback
